<?php

namespace goodboyalex\php_components_pack\classes;

use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;

/**
 * Класс, реализующий функционал работы с файлами и выполнение операций над файлами одной командой.
 *
 * @author Александр Бабаев
 * @package php_components_pack
 * @version 1.0.1
 * @since 1.0.21
 */
final class File
{
    /**
     * Получает список файлов в директории и поддиректориях, соответствующей шаблону $pattern.
     *
     * @param string $dir Родительская директория
     * @param string $pattern Шаблон имени файла
     *
     * @return false|array Список файлов или false в случае ошибки
     */
    public static function FindFiles (string $dir, string $pattern = '*.php'): false|array
    {
        // Получаем список файлов и каталогов в текущей директории
        $files = glob("$dir/$pattern");

        // Если произошла ошибка
        if ($files === false)
            // - то возвращаем false
            return false;

        // Перебираем поддиректории
        foreach (glob("$dir/*", GLOB_ONLYDIR | GLOB_NOSORT) as $subDir) {
            // - если ошибка
            if ($subDir === false)
                // - то пропускаем
                continue;

            // - рекурсивный вызов для каждой поддиректории
            $files = array_merge($files, self::FindFiles($subDir, $pattern));
        }

        // Возвращаем список файлов
        return $files;
    }

    /**
     * Получает имя файла без пути к нему и расширения.
     *
     * @param string $fileName Полное имя файла с путем к нему.
     *
     * @return string Имя файла без пути к нему и расширения.
     */
    public static function ExtractFileNameWithoutExtension (string $fileName): string
    {
        // Имя файла без пути к нему
        $fileNameOnly = self::ExtractFileName($fileName);

        // Расширение файла
        $fileExtension = self::ExtractFileExtension($fileName);

        // Возвращаем имя файла без пути к нему и расширения.
        return substr($fileNameOnly, 0, -strlen($fileExtension) - 1);
    }

    /**
     * Получает имя файла без пути к нему, но с расширением.
     *
     * @param string $fileName Полное имя файла с путем к нему.
     *
     * @return string Имя файла без пути к нему, но с расширением.
     */
    public static function ExtractFileName (string $fileName): string
    {
        return basename($fileName);
    }

    /**
     * Получает расширение файла.
     *
     * @param string $fileName Имя файла с путем к нему.
     *
     * @return string Расширение файла.
     */
    public static function ExtractFileExtension (string $fileName): string
    {

        return pathinfo($fileName, PATHINFO_EXTENSION);
    }

    /**
     * Получает относительный путь к файлу, относительно заданной папки
     *
     * @param string $fullPath Полный путь к файлу
     * @param string $basePath Вырезаемый путь (с начала)
     *
     * @return false|string Относительный путь к файлу
     */
    public static function GetRelativePath (string $fullPath, string $basePath): false|string
    {
        return stripos($fullPath, $basePath) !== false ? str_replace($basePath, "", $fullPath) : false;
    }

    /**
     * Удаляет директорию вместе со всеми файлами и поддиректориями.
     *
     * @param string $directory Полный путь к директории.
     *
     * @return bool Результат удаления.
     */
    public static function RemoveDir (string $directory): bool
    {
        // Проверяем, существует ли директория
        if (!self::DirectoryExists(directory: $directory, checkWriteAccess: true))
            // - если нет, то возвращаем false
            return false;

        // Получаем список файлов и каталогов в заданной директории
        $files = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($directory),
            RecursiveIteratorIterator::CHILD_FIRST
        );

        // Перебираем файлы и каталоги
        foreach ($files as $file)
            // - если это каталог
            if ($file->isDir())
                // -- то удаляем его
                rmdir($file->getRealPath());
            else
                // -- иначе удаляем файл
                unlink($file->getRealPath());

        // Удаляем директорию
        return rmdir($directory);
    }

    /**
     * Проверяет, существует ли директория.
     *
     * @param string $directory Путь к директории.
     * @param bool $checkReadAccess Проверять ли доступ на чтение директории (по умолчанию true).
     * @param bool $checkWriteAccess Проверять ли доступ на запись директории (по умолчанию false).
     *
     * @return bool Результат проверки.
     */
    public static function DirectoryExists (string $directory, bool $checkReadAccess = true,
        bool $checkWriteAccess = false): bool
    {
        // Очищаем кэш
        clearstatcache();

        // Проверяем, существует ли директория
        if (!file_exists($directory))
            // - если нет, то возвращаем false
            return false;

        // Проверяем, является ли директория директорией, а не файлом
        if (!is_dir($directory))
            // - если нет, то возвращаем false
            return false;

        // Проверяем, есть ли доступ на чтение директории
        if ($checkReadAccess && !is_readable($directory))
            // - если нет, то возвращаем false
            return false;

        // Проверяем, есть ли доступ на запись директории
        if ($checkWriteAccess && !is_writable($directory))
            // - если нет, то возвращаем false
            return false;

        // Если все проверки пройдены успешно, то возвращаем true
        return true;
    }
}